// Source file for the GLUT programs


// RING include files

#include "R2Shapes/R2Shapes.h"



// OpenGL include files 

#include <GL/glut.h>



// GLUT variables 

static int GLUTwindow = 0;
static int GLUTwindow_height = 480;
static int GLUTwindow_width = 640;
static int GLUTmouse[2] = { 0, 0 };
static int GLUTbutton[3] = { 0, 0, 0 };
static int GLUTarrows[4] = { 0, 0, 0, 0 };
static int GLUTmodifiers = 0;



// Display variables

static int scaling = 0;
static int translating = 0;
static float scale = 1.0;
static float translation[2] = { 0.0, 0.0 };
static int show_wireframe = 0;



// Program variables

static RNArray<R2Shape *> shapes;



void GLUTStop(void)
{
  // Destroy window 
  glutDestroyWindow(GLUTwindow);

  // Exit
  exit(0);
}



void GLUTRedraw(void)
{
  // Clear window 
  glClear(GL_COLOR_BUFFER_BIT);

  // Setup viewing transformation
  glLoadIdentity();
  glScalef(scale, scale, scale);
  glTranslatef(translation[0], translation[1], 0.0);

  // Draw shapes 
  glColor3f(1.0, 1.0, 1.0);
  for (int i = 0; i < shapes.NEntries(); i++) {
    R2Shape *shape = shapes.Kth(i);
    if (show_wireframe) shape->Outline();
    else shape->Draw();
  }
    
  // Swap buffers 
  glutSwapBuffers();
}    



void GLUTResize(int w, int h)
{
  // Resize window
  glViewport(0, 0, w, h);

  // Remember window size 
  GLUTwindow_width = w;
  GLUTwindow_height = h;

  // Redraw
  GLUTRedraw();
}



void GLUTMotion(int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;
  
  // Process mouse motion event
  if (scaling) {
    // Scale window
    scale *= exp(-2.0 * (float) (x - GLUTmouse[0]) / (float) GLUTwindow_width);
  }
  else if (translating) {
    // Translate window
    translation[0] += 1.0 * (float) (x - GLUTmouse[0]) / (float) GLUTwindow_width;
    translation[1] += 1.0 * (float) (y - GLUTmouse[1]) / (float) GLUTwindow_height;
  }

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;
}



void GLUTMouse(int button, int state, int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;
  
  // Process mouse button event
  scaling = (button == GLUT_RIGHT_BUTTON);
  translating = (button == GLUT_MIDDLE_BUTTON);
  if (scaling || translating) glutIdleFunc(GLUTRedraw);
  else glutIdleFunc(0);

  // Remember button state 
  int b = (button == GLUT_LEFT_BUTTON) ? 0 : ((button == GLUT_MIDDLE_BUTTON) ? 1 : 2);
  GLUTbutton[b] = (state == GLUT_DOWN) ? 1 : 0;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

   // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;
}



void GLUTSpecial(int key, int x, int y)
{
  // Invert y coordinate
  y = GLUTwindow_height - y;

  // Process keyboard button event 

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = y;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

  // Redraw
  GLUTRedraw();
}



void GLUTKeyboard(unsigned char key, int x, int y)
{
  // Flag to determine whether to redraw
  int redraw = 0;

  // Process keyboard button event 
  switch (key) {
  case 'W':
  case 'w':
    show_wireframe = !show_wireframe;
    break;
      
  case 27: // ESCAPE
    GLUTStop();
    break;
  }

  // Redraw 
  if (redraw) GLUTRedraw();

  // Remember mouse position 
  GLUTmouse[0] = x;
  GLUTmouse[1] = GLUTwindow_height - y;

  // Remember modifiers 
  GLUTmodifiers = glutGetModifiers();

  // Redraw
  GLUTRedraw();
}



int GLUTInit(int *argc, char **argv)
{
  // Background color
  float background[3] = { 0.0, 0.0, 0.0 }; 

  // Open window 
  glutInit(argc, argv);
  glutInitWindowPosition(100, 100);
  glutInitWindowSize(GLUTwindow_width, GLUTwindow_height);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); // | GLUT_STENCIL
  GLUTwindow = glutCreateWindow("RING Viewer");

  // Setup GLUT callback functions 
  glutReshapeFunc(GLUTResize);
  glutDisplayFunc(GLUTRedraw);
  glutKeyboardFunc(GLUTKeyboard);
  glutSpecialFunc(GLUTSpecial);
  glutMouseFunc(GLUTMouse);
  glutMotionFunc(GLUTMotion);
  glutIdleFunc(0);
    
  // Set background color 
  glClearColor(background[0], background[1], background[2], 1.0);

  // Set viewing transformation 
  glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

  // Return OK status 
  return 1;
}



void GLUTMainLoop(void)
{
  // Run main loop -- never returns 
  glutMainLoop();
}



static int CreateShapes(void)
{
  // Create a circle
  R2Circle *c1 = new R2Circle(R2zero_point + R2posy_vector * 0.25, 0.25);
  shapes.Insert(c1);

  // Create a box
  R2Box *box1 = new R2Box(0.25, -0.75, 0.75, -0.25);
  shapes.Insert(box1);
  
  // Return success
  return 1;
}



int 
main(int argc, char **argv)
{
  // Initialize RING
  if (!R2InitShapes()) exit(1);

  // Initialize GLUT interface
  if (!GLUTInit(&argc, argv)) exit(1);

  // Create shapes
  if (!CreateShapes()) exit(1);

  // Run the interactive interface 
  GLUTMainLoop();

  // Stop GLUT interface
  GLUTStop();

  // Stop RING
  R2StopShapes();

  // Return success 
  return 0;
}











